// The following code is example code from 
// "ActionScript 3.0 Bible, 2nd Edition" by Roger Braunstein
// Find all of the source code on the companion website
// http://actionscriptbible.com/
//
// Copyright (c) 2010 by Wiley Publishing, Inc., Indianapolis, Indiana
//
// Permission is hereby granted, free of charge, to any person
// obtaining a copy of this software and associated documentation
// files (the "Software"), to deal in the Software without
// restriction, including without limitation the rights to use,
// copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following
// conditions:
//
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
// OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
// WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
// OTHER DEALINGS IN THE SOFTWARE.

package {
  import flash.display.Sprite;
  import flash.events.MouseEvent;
  import flash.filters.DropShadowFilter;
  
  public class ch14ex2 extends Sprite {
    protected const NUM_FILES:int = 10;
    protected var deleteBin:Sprite;
    public function ch14ex2() {
      deleteBin = makeDeleteBin();
      //the delete bin should stay at the bottom
      addChildAt(deleteBin, 0);
      deleteBin.x = 15;
      deleteBin.y = 15;
      
      for (var i:int = 0; i < NUM_FILES; i++) {
        var file:Sprite = makeFile();
        addChild(file);
        //randomize position by looking at available stage size
        file.x = Math.random() * (stage.stageWidth - file.width);
        file.y = Math.random() * (stage.stageHeight - file.height);
        //Sprites are InteractiveObjects
        file.addEventListener(MouseEvent.MOUSE_DOWN, onFileMouseDown);
        file.addEventListener(MouseEvent.MOUSE_UP, onFileMouseUp);
      }
    }
    protected function onFileMouseDown(event:MouseEvent):void {
      var file:Sprite = Sprite(event.target);
      file.startDrag(); //Sprites have simple drag methods
      //moving is relative change in position
      file.x -= 2;
      file.y -= 2;
      //all DisplayObjects support filters
      file.filters = [new DropShadowFilter(2, 45, 0, 0.2)]; 
      setChildIndex(file, numChildren-1); //set child depth to the top
    }
    protected function onFileMouseUp(event:MouseEvent):void {
      var file:Sprite = Sprite(event.target);
      file.stopDrag(); //Sprites have simple drag methods
      file.x += 2;
      file.y += 2;
      file.filters = [];
      //see if it's over the delete bin
      if (deleteBin.hitTestObject(file)) {
        //and if so, remove from display list
        removeChild(file);
      }
    }
    protected function makeDeleteBin():Sprite {
      var s:Sprite = new Sprite();
      //Sprites support vector drawing
      s.graphics.beginFill(0xff0000);
      s.graphics.drawRoundRect(0, 0, 55, 70, 16);
      s.graphics.endFill();
      return s;
    }
    protected function makeFile():Sprite {
      var s:Sprite = new Sprite();
      //Sprites support vector drawing
      s.graphics.beginFill(0xc0c0c0);
      s.graphics.lineStyle(0, 0x808080);
      s.graphics.drawRect(0, 0, 8.5, 11);
      s.graphics.endFill();
      s.scaleX = s.scaleY = 3; //DisplayObjects support scaling
      s.buttonMode = true; //Sprites can act like buttons
      return s;
    }
  }
}