// The following code is example code from 
// "ActionScript 3.0 Bible, 2nd Edition" by Roger Braunstein
// Find all of the source code on the companion website
// http://actionscriptbible.com/
//
// Copyright (c) 2010 by Wiley Publishing, Inc., Indianapolis, Indiana
//
// Permission is hereby granted, free of charge, to any person
// obtaining a copy of this software and associated documentation
// files (the "Software"), to deal in the Software without
// restriction, including without limitation the rights to use,
// copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following
// conditions:
//
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
// OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
// WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
// OTHER DEALINGS IN THE SOFTWARE.

package {
  import flash.display.*;
  import flash.events.Event;
  import flash.geom.Point;
  import flash.geom.Rectangle;
  import flash.net.URLLoader;
  import flash.net.URLRequest;
  
  public class ch36ex17 extends Sprite {
    protected var map:Vector.<Vector.<int>>;
    protected var tilesize:int;
    protected var library:Vector.<BitmapData>;
    protected var onscreen:BitmapData;
    protected var bitmap:Bitmap;
    protected const PATH:String = "../etc/";
    public function ch36ex17() {
      var u:URLLoader = new URLLoader(new URLRequest(PATH + "map.xml"));
      u.addEventListener(Event.COMPLETE, onMapLoad);
    }
    protected function onMapLoad(event:Event):void {
      XML.ignoreWhitespace = false;
      var xml:XML = XML(URLLoader(event.target).data);
      tilesize = parseInt(xml.art.@basesize);
      var rows:int = parseInt(xml.map.@rows);
      var cols:int = parseInt(xml.map.@cols);
      var textData:String = xml.map.text();
      var textRows:Array = textData.split(/[\r|\n]+/);
      map = new Vector.<Vector.<int>>(rows, true);
      for (var row:int = 0; row < rows; row++) {
        map[row] = new Vector.<int>(cols, true);
        var textCols:Array = textRows[row].split(/\s*,\s*/);
        for (var col:int = 0; col < cols; col++) {
          map[row][col] = parseInt(textCols[col]);
        }
      }
      onscreen = new BitmapData(tilesize * map[0].length,
        tilesize * map.length, false, parseInt(xml.map.@bgcolor, 16));
      bitmap = new Bitmap(onscreen, PixelSnapping.ALWAYS, false);
      bitmap.scaleX = bitmap.scaleY = 2;
      addChild(bitmap);
      
      //load images
      var loader:Loader = new Loader();
      loader.load(new URLRequest(PATH + xml.art.@src));
      loader.contentLoaderInfo.addEventListener(Event.COMPLETE, onArtLoad);
    }
    protected function onArtLoad(event:Event):void {
      var src:BitmapData = Bitmap(LoaderInfo(event.target).content).bitmapData;
      var cols:int = src.width / tilesize;
      var rows:int = src.height / tilesize;
      library = new Vector.<BitmapData>();
      for (var row:int = 0, col:int = 0, i:int = 0; i < rows*cols; i++) {
        library[i] = new BitmapData(tilesize, tilesize);
        library[i].copyPixels(src, new Rectangle(col * tilesize, row * tilesize, tilesize, tilesize), new Point());
        if (++col > cols) {
          col = 0;
          row++;
        }
      }
      buildMap();
    }
    protected function buildMap():void {
      var tileRect:Rectangle = new Rectangle(0, 0, tilesize, tilesize);
      for (var y:int = 0; y < map.length; y++) {
        for (var x:int = 0; x < map[0].length; x++) {
          var tileid:int = map[y][x];
          if (tileid > -1) {
            onscreen.copyPixels(library[map[y][x] - 1], tileRect, new Point(x*tilesize, y*tilesize));
          }
        }
      }
    }
  }
}